module  climat_forcage_mod

! Forcage climatique avec climat GCM et fichier de forcage temporel index 
! lecture de fichiers Snapshots climatiques  temps t
! lecture d'un fichier de forcage (temp carotte de glace)
! possibilite d'utiliser autant de snapshots que l'on veut (ntr)
! fichiers de forcage, Snapshot et valeurs de lapse rate definis dans fichier param
  use module3d_phy,only: sealevel,sealevel_2d,S,Tmois,Tann,Tjuly,acc,num_forc,num_param,num_rep_42,tafor,coefbmshelf,PI,time
  use geography, only: nx,ny,dirforcage,dirnameinp
  use param_phy_mod,only:ro
  use io_netcdf_grisli, only: read_ncdf_var


  implicit none
  character(len=100) :: filin
  integer :: nft             !   nft est le nombre de lignes a lire dans le fichier 
                           ! contenant le forcage climatique
  real,dimension(:),allocatable :: tdate          ! time for climate forcing
  real,dimension(:),allocatable :: tpert
  real,dimension(:),allocatable :: spert
  real,dimension(:),allocatable :: bpert
  
  integer :: ntr         ! nb of snapshot files now explicitely specified 

  character(len=100) :: clim_ref_file ! climat de reference
  character(len=70), dimension(5) :: forcage_file ! fichier de forcage (climat+topo) => dimension >= ntr
!  character(len=100) :: forcage_file2 ! fichier de forcage 2 (climat+topo)

  character(len=100) :: filforc       ! nom du fichier forcage
  real :: coefT                    !< pour modifier l'amplitude de la perturb. T
  integer :: pertbmb               !< boolean: do we modify the bmelt under ice shelves?
  real    :: coefbmb               !< if we modify the bmelt, we read a perturbation (based on dD, insolation, etc.) and apply this coefficient
  real    :: r_atmvar              !< a ratio to account fast atmos variability between the 2 snapshots (in %: 0=>no fast var,1=>only fast var)



!  character(len=200) :: filtr(ntr)   ! fichiers de forcage

  real,dimension(:), allocatable :: ttr         !< date des tranches (snapshots) 
  real,dimension(nx,ny) :: delTatime, delTjtime, rapactime
  real,dimension(nx,ny) :: Zs        !< surface topography above sea level

  integer :: typerun ! 1 for steady state using snap 1, 
                     ! 2 for steady state using snap 2 ...
                     ! 0 for transient
  
! variables pour climato mensuelle :  
  real,dimension(nx,ny,12) :: t2m0   !< initial monthly air temperature (climato)
  real,dimension(nx,ny,12) :: pr0    !< initial monthly precip (climato)
  real,dimension(nx,ny) :: orog0     !< topo de la climato
  
! climat des snapshots :
  real,dimension(:,:,:,:), allocatable :: t2m_ss  !< t2m monthly for every GCM snapshot 
  real,dimension(:,:,:,:), allocatable :: pr_ss   !< precip monthly for every GCM snapshot
  real,dimension(:,:,:), allocatable :: orog_ss    !< topo for every GCM snapshot 
  
  real,dimension(:,:,:,:), allocatable :: t2m_sstopo0 !< t2m GCM snapshot on orog0
  real,dimension(:,:,:,:), allocatable :: pr_sstopo0  !< pr GCM snapshot on orog0


!~   real,dimension(nx,ny,12,ntr) :: t2m_ss  !< t2m monthly for every GCM snapshot 
!~   real,dimension(nx,ny,12,ntr) :: pr_ss   !< precip monthly for every GCM snapshot
!~   real,dimension(nx,ny,ntr) :: orog_ss    !< topo for every GCM snapshot 
  
!~   real,dimension(nx,ny,12,ntr) :: t2m_sstopo0 !< t2m GCM snapshot on orog0
!~   real,dimension(nx,ny,12,ntr) :: pr_sstopo0  !< pr GCM snapshot on orog0
  
  real :: PYY                        !< constante pour calcul de la fraction solide des precips
  real :: psolid=2.                  !< temp limit between liquid and solid precip
  real,dimension(nx,ny) :: FT        !< proportion of year with air temp below psolid
  real :: lapserate                  !< gradient vertical de temp annuel et juillet
  real :: rappact                    !< coef pour calcul du rapport accumulation
  logical :: annual                  !< True si forcage annuel, False si forcage mensuel

contains

!------------------------------------------------------------------------------------------
subroutine input_clim          !routine qui permet d'initialiser les variables climatiques
    
  implicit none
  character(len=8) :: control     !< label to check clim. forc. file (filin) is usable
  integer :: l                    !< in snapshot files:the first column is the mask, read but not used
  integer :: err                  !< recuperation erreur
  integer :: i,j,k,m
  real*8, dimension(:,:), pointer   :: tab2d => null()   !< tableau 2d pointer
  real*8, dimension(:,:,:), pointer :: tab3d => null()   !< tableau 3d pointer

  deltatime(:,:)=0.
  deltjtime(:,:)=0.
  rapactime(:,:)=1.
  annual=.true.  ! forcage avec fichier Tann et Tjuly

! lecture du climat de reference : climato mensuelle
  print*,'lecture du climat de reference: ',trim(DIRNAMEINP)//TRIM(clim_ref_file)
  call Read_Ncdf_var('precip',trim(DIRNAMEINP)//TRIM(clim_ref_file),tab3d)
  pr0(:,:,:)=tab3d(:,:,:)
  call Read_Ncdf_var('t2m',trim(DIRNAMEINP)//TRIM(clim_ref_file),tab3d)
  t2m0(:,:,:)=tab3d(:,:,:)
  call Read_Ncdf_var('orog',trim(DIRNAMEINP)//TRIM(clim_ref_file),tab2d)
  orog0(:,:)=tab2d(:,:)
  
!  filtr(1)=TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file1)
!  filtr(2)=TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file2)

! fichiers donnant l'evolution temporelle
! -----------------------------------------
! Pour rappel hemin40 :
! fichier forcage : signal-cycle-hemin.dat
! Pour anteis :
! fichier forcage : forcmodif4cycles.dat
  filin=trim(dirforcage)//trim(filforc) 


! lecture des fichiers snapshots pour n'importe quel geoplace
! -------------------------------------------------------------
  do k=1,ntr
     write(6,*) 'Read climate file :',trim(DIRNAMEINP)//'Snapshots-GCM/'//trim(forcage_file(k))
     call Read_Ncdf_var('pr',trim(DIRNAMEINP)//'Snapshots-GCM/'//trim(forcage_file(k)),tab3d)
     pr_ss(:,:,:,k)=tab3d(:,:,:)
     call Read_Ncdf_var('tas',trim(DIRNAMEINP)//'Snapshots-GCM/'//trim(forcage_file(k)),tab3d)
     t2m_ss(:,:,:,k)=tab3d(:,:,:)
     call Read_Ncdf_var('orog',trim(DIRNAMEINP)//'Snapshots-GCM/'//trim(forcage_file(k)),tab2d)
     orog_ss(:,:,k)=tab2d(:,:)
!~      read(num_forc,*) ttr(k)
!~      read(num_forc,*)
!~      read(num_forc,*)
!~      do j=1,ny
!~         do i=1,nx
!~            read(num_forc,*) l,rapact(i,j,k),delta(i,j,k),deltj(i,j,k)
!~         end do
!~      end do
!~      close(num_forc)
  end do


! calcul de t2m et pr sur la topo des obs :
  do m=1,12
    do k=1,ntr
      t2m_sstopo0(:,:,m,k)=t2m_ss(:,:,m,k)-lapserate*(orog0(:,:)-orog_ss(:,:,k))
      pr_sstopo0(:,:,m,k)=pr_ss(:,:,m,k)*exp(rappact*(t2m_sstopo0(:,:,m,k)-t2m_ss(:,:,m,k)))
    enddo
  enddo


! lecture des fichiers d'evolution pour tout geoplace
! ----------------------------------------------------
  open(num_forc,file=filin,status='old')
!        print*,nft
  read(num_forc,*) control,nft
!  print*,'control :',control,nft
! determination of file size (line nb), allocation of perturbation array

  if (control.ne.'nb_lines') then
     write(6,*) filin,'indiquer le nb de ligne en debut de fichier:'
     write(6,*) 'le nb de lignes et le label de control nb_lines'
     stop
  endif

  if (.not.allocated(tdate)) THEN
     allocate(tdate(nft),stat=err)
     if (err/=0) then
        print *,"erreur  l'allocation du tableau tdate",err
        stop 4
     end if
  end if

  if (.not.allocated(tpert)) THEN
     allocate(tpert(nft),stat=err)
     if (err/=0) then
        print *,"erreur  l'allocation du tableau tpert",err
        stop 4
     end if
  end if

  if (.not.allocated(bpert)) THEN
     allocate(bpert(nft),stat=err)
     if (err/=0) then
        print *,"erreur  l'allocation du tableau tpert",err
        stop 4
     end if
  end if

  if (.not.allocated(spert)) THEN
     allocate(spert(nft),stat=err)
     if (err/=0) then
        print *,"erreur  l'allocation du tableau spert",err
        stop 4
     end if
  end if

  do i=1,nft
     if (pertbmb==1) then
        read(num_forc,*) tdate(i),spert(i),tpert(i),bpert(i)
     else
        read(num_forc,*) tdate(i),spert(i),tpert(i)
        bpert(i)=0.
     endif
  end do

  close(num_forc)

! Warning: when using climat_forcage we should in principle use a tpert which is a glacial index (ranging from about 0 to 1)
!          here we should use the coefT to scale the signal if needed
!          the coefbmb now is copy and paste from what we have done for Antarctica in Quiquet et al. GMD 2018

  tpert(:)=tpert(:)*coefT
  bpert(:)=max(1.+bpert(:)*coefbmb,0.01) !freezing is not allowed



end subroutine input_clim


!--------------------------------------------------------------------------------
subroutine init_forclim

  real,dimension(5) :: ttr_temp         !< date des tranches (snapshots) 
  integer :: err                  !< recuperation erreur
  integer :: k
  
  namelist/clim_forcage/clim_ref_file,ntr,ttr_temp,forcage_file,typerun,lapserate,rappact,filforc,pertbmb,coeft,coefbmb,r_atmvar

  rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
  read(num_param,clim_forcage)
  
  if (ntr.le.size(ttr_temp,dim=1)) then
   print*,'climat_forcage : we will read ',ntr,' climate snapshots'
   do k=ntr+1,size(ttr_temp,dim=1)
      ttr_temp(k)= ttr_temp(ntr)
      forcage_file(k)=' '
   enddo
  else
   print*,'climat_forcage ERROR : ntr > ttr_temp size'
   print*,'ttr_temp & forcage_file size must be >= ntr'
   stop
  end if
  
  write(num_rep_42,'(A)')'!___________________________________________________________' 
  write(num_rep_42,clim_forcage)


  
  if (.not.allocated(ttr)) THEN
   allocate(ttr(ntr),stat=err)
   if (err/=0) then
      print *,"Erreur  l'allocation du tableau ttr",err
      stop 4
   end if
  end if

  if (.not.allocated(t2m_ss)) THEN
   allocate(t2m_ss(nx,ny,12,ntr),stat=err)
   if (err/=0) then
      print *,"Erreur  l'allocation du tableau t2m_ss",err
      stop 4
   end if
  end if
  
  if (.not.allocated(pr_ss)) THEN
   allocate(pr_ss(nx,ny,12,ntr),stat=err)
   if (err/=0) then
      print *,"Erreur  l'allocation du tableau pr_ss",err
      stop 4
   end if
  end if
  
  if (.not.allocated(orog_ss)) THEN
   allocate(orog_ss(nx,ny,ntr),stat=err)
   if (err/=0) then
      print *,"Erreur  l'allocation du tableau orog_ss",err
      stop 4
   end if
  end if 
  
  if (.not.allocated(t2m_sstopo0)) THEN
   allocate(t2m_sstopo0(nx,ny,12,ntr),stat=err)
   if (err/=0) then
      print *,"Erreur  l'allocation du tableau t2m_sstopo0",err
      stop 4
   end if
  end if
  
  if (.not.allocated(pr_sstopo0)) THEN
   allocate(pr_sstopo0(nx,ny,12,ntr),stat=err)
   if (err/=0) then
      print *,"Erreur  l'allocation du tableau pr_sstopo0",err
      stop 4
   end if
  end if
  
  do k=1,ntr
   ttr(k)=ttr_temp(k)
  enddo
   

!~   write(num_rep_42,'(A)')'!___________________________________________________________' 
!~   write(num_rep_42,'(A)') '&clim_forcage                                ! nom du bloc '
!~   write(num_rep_42,*)
!~   write(num_rep_42,'(A,A,A)') 'clim_ref_file = "',trim(clim_ref_file),'"'
!~   write(num_rep_42,*) 'ttr = ', ttr
!~ !  write(num_rep_42,'(A,A,A)') 'forcage_file= "',trim(forcage_file),'"'
!~   write(num_rep_42,'(A,A,A)') 'forcage_file1 = "',trim(forcage_file1),'"'
!~   write(num_rep_42,'(A,A,A)') 'forcage_file2 = "',trim(forcage_file2),'"'
!~   write(num_rep_42,*) 'typerun = ', typerun
!~   write(num_rep_42,*) 'lapserate = ', lapserate
!~   write(num_rep_42,*) 'rappact = ', rappact
!~   write(num_rep_42,'(A,A,A)') ' filforc      = "',trim(filforc),'"'
!~   write(num_rep_42,*) 'pertbmb = ', pertbmb
!~   write(num_rep_42,*) 'coefT   = ', coefT
!~   write(num_rep_42,*) 'coefbmb = ', coefbmb
!~   write(num_rep_42,*) 'r_atmvar = ', r_atmvar
!~   write(num_rep_42,*)'/'                            
!~   write(num_rep_42,*)

  PYY=2.*PI/50.

end subroutine init_forclim


!---------------------------------------------------------------------
!forcage climatique au cours du temps

subroutine forclim

!$ USE OMP_LIB

  implicit none
  real :: coeftime       !< coeftime is not used
  real :: tafor
  integer :: l                       !< dumm index for loops on snapsots files  l=itr,ntr-1
  integer :: itr                     !< nb of the current snapshot file (change with time)
  integer :: i,j,k,m
  integer :: ift                     !< indice correspondant au pas de temps du fichier de forcage
  real :: TEMP                       !< calcul du nbr de jour < psolid
  real,dimension(nx,ny) :: deltaZs   ! diff temp par rapport a topo orog0 
  real,dimension(nx,ny,12) :: prmois ! precip sur topo Zs
  
  real (kind=kind(0.d0)) ::  timeloc
  
  if (typerun.eq.0) then
     timeloc = time
  else if ( (typerun .gt. 0) .and. (typerun .le. ubound(ttr,dim=1)) ) then
     timeloc = ttr(typerun)
  else
     write(*,*) "Unknown type of simulation for climat_forcage, abort"
     STOP
  end if

  if(timeloc.le.tdate(1)) then   ! time avant le debut du fichier forcage
     sealevel=spert(1)
     tafor=tpert(1)
     coefbmshelf=bpert(1)
     ift=1

  else if (timeloc.ge.tdate(nft)) then     ! time apres la fin du fichier forcage
     sealevel=spert(nft)
     tafor=tpert(nft)
     coefbmshelf=bpert(nft)
     ift=nft

  else                         ! time en general

     ift = 1      
     do i=ift,nft-1  ! interpolation entre i et i+1

        if((timeloc.ge.tdate(i)).and.(timeloc.lt.tdate(i+1))) then  ! cas general
           sealevel=spert(i)+(spert(i+1)-spert(i))*    &
                (timeloc-tdate(i))/(tdate(i+1)-tdate(i))

           tafor=tpert(i)+(tpert(i+1)-tpert(i))*      &
                (timeloc-tdate(i))/(tdate(i+1)-tdate(i))

           coefbmshelf=bpert(i)+(bpert(i+1)-bpert(i))*      &
                (timeloc-tdate(i))/(tdate(i+1)-tdate(i))
                
           ift=i
           goto 100
        endif
     end do
  endif
100 continue


!same as for the GMD2018 paper:
!afq -- as in Pollard and Deconto, melt is more efficient at high temperature... ->
  if (coefbmshelf.gt.1.) then
     coefbmshelf = 1. + ( coefbmshelf - 1. ) * 1.5 !afq.... tuning for the degla?...
  end if
       
  sealevel_2d(:,:)=sealevel

  if(timeloc.le.ttr(1)) then   ! time en dehors des limites du fichier forcage
    coeftime=0.
!~     do j=1,ny
!~       do i=1,nx
!~         deltatime(i,j)=delta(i,j,1)
!~         deltjtime(i,j)=deltj(i,j,1)
!~         rapactime(i,j)=rapact(i,j,1)
!~       end do
!~     end do
    itr=1
  else if (timeloc.ge.ttr(ntr)) then        ! mis a 0
    coeftime=1.
    itr=ntr
!~     do j=1,ny
!~       do i=1,nx
!~         deltatime(i,j)=delta(i,j,ntr)
!~         deltjtime(i,j)=deltj(i,j,ntr)
!~         rapactime(i,j)=rapact(i,j,ntr)
!~       end do
!~     end do
  else               !  interpolation entre l et l+1 : cas general
     itr=1
     do l=itr,ntr-1    ! interpolation entre i et i+1
        if((timeloc.ge.ttr(l)).and.(timeloc.lt.ttr(l+1))) then  ! test tranche
           coeftime= (timeloc-ttr(l))/(ttr(l+1)-ttr(l))
!           do j=1,ny
!              do i=1,nx
!                 delTatime(i,j)=delta(i,j,l)+             &
!                      (delta(i,j,l+1)-delta(i,j,l))*coeft
!                 delTjtime(i,j)=deltj(i,j,l)+             &
!                      (deltj(i,j,l+1)-deltj(i,j,l))*coeft
!                 rapactime(i,j)=rapact(i,j,l)+            &
!                      (rapact(i,j,l+1)-rapact(i,j,l))*coefp
!              end do
!           end do
           itr=l
        endif        ! fin du test sur la tranche
     end do
  endif   ! fin du test avant,apres,milieu

  if (abs(Tafor).gt.2.) then
     write(*,*) "A glacial index greater than 2? Really??"
     STOP
  endif
  coeftime = (1-r_atmvar)*coeftime + r_atmvar*(1-Tafor)

!if (geoplace(1:5).eq.'hemin') then

  !$OMP PARALLEL
!     calcul de Tjuly et et Tann
  !$OMP WORKSHARE
  Zs(:,:)=max(sealevel_2d(:,:),S(:,:))
  deltaZs(:,:)= -lapserate*(Zs(:,:)-orog0(:,:)) ! difference de temperature entre Zs et orog0
  !$OMP END WORKSHARE
! correction d'altitude
do m=1,12 
  if ((itr.LT.ntr-1).and.(ntr.ge.3)) then
! if the is more than 2 snapshots : time climate is mean between 2 snapshots and anomaly with snapshot ntr (ctrl)  
    !$OMP WORKSHARE
    Tmois(:,:,m)=  (t2m_sstopo0(:,:,m,itr)*(1-coeftime) + t2m_sstopo0(:,:,m,itr+1)*coeftime) - t2m_sstopo0(:,:,m,ntr) + t2m0(:,:,m) + deltaZs(:,:)
    prmois(:,:,m)= pr0(:,:,m)*((pr_sstopo0(:,:,m,itr)*(1-coeftime) + pr_sstopo0(:,:,m,itr+1)*coeftime)/ pr_sstopo0(:,:,m,ntr)) * exp(rappact*(deltaZs(:,:)))
    !$OMP END WORKSHARE

! Temp et precip fonction de coeftime :
  else if (itr.LT.ntr) then
    !$OMP WORKSHARE
    Tmois(:,:,m)=  (t2m_sstopo0(:,:,m,itr) - t2m_sstopo0(:,:,m,itr+1))*(1-coeftime) + t2m0(:,:,m) + deltaZs(:,:)
    prmois(:,:,m)= pr0(:,:,m)*(coeftime+(1-coeftime)*(pr_sstopo0(:,:,m,itr)/pr_sstopo0(:,:,m,itr+1))) * exp(rappact*(deltaZs(:,:)))
    !$OMP END WORKSHARE
  else ! itr=ntr (time>tsnapmax)
    !$OMP WORKSHARE
    Tmois(:,:,m)=  t2m0(:,:,m) + deltaZs(:,:)
    prmois(:,:,m)= pr0(:,:,m)*exp(rappact*(deltaZs(:,:)))
    !$OMP END WORKSHARE
  endif            
enddo 



  !$OMP WORKSHARE
  Tann=sum(Tmois,dim=3)/12.
  Tjuly=Tmois(:,:,7)
  



! calcul de l'accumulation de neige :
  acc(:,:)=sum(prmois,dim=3,mask=Tmois < psolid)
  acc(:,:)=acc(:,:)*1000./ro  ! conversion en glace
  !$OMP END WORKSHARE
  !$OMP END PARALLEL
!debug
!print*,'Tann, Tjuly',Tann(142,14),Tjuly(142,14)  
!print*,'acc',acc(142,14)

end subroutine forclim

end module  climat_forcage_mod
