!> \file climat_smb_perturb_mod.f90
!! Direct SMB forcing + perturbation with glacial index
!<

!> \namespace  climat_smb_perturb_mod
!! Direct SMB forcing + perturbation with glacial index
!! \author Many
!! \date old
!! @note Used modules:
!! @note   - use module3D_phy
!<
module climat_smb_perturb_mod

use geography, only: nx,ny

implicit none

real :: T_lapse_rate              !< pour la temperature

real,dimension(nx,ny)                :: bm_0              !> bilan de masse initial equ. S_0ref de Tamsin
real                                 :: coef_smb_unit     ! pour corriger l'unite
real,dimension(nx,ny)                :: TA0               !> Temp annuelle sur S0


! aurel, pour climat type perturb:
integer nft             !> nombre de lignes a lire dans le fichier forcage climatique
real,dimension(:),allocatable :: tdate          !> time for climate forcing
real,dimension(:),allocatable :: tpert          !> temperature for climate forcing
real,dimension(:),allocatable :: spert          !> sea surface perturbation
real,dimension(:),allocatable :: bpert          !> basal melt index perturbation
real :: coefT                    !> pour modifier l'amplitude de la perturb. T
character(len=150) :: filforc    !> nom du fichier forcage
integer :: pertsmb               !> boolean: do we modify the smb?
real    :: rapsmb                !> if we modify the smb this is the equivalent of rappact
integer :: pertbmb               !> boolean: do we modify the bmelt under ice shelves?
real    :: coefbmb               !> if we modify the bmelt, we read a perturbation (based on dD, insolation, etc.) and apply this coefficient


! pour les lectures ncdf
real*8, dimension(:,:,:),  pointer   :: tab3D             !< tableau 3d real pointer
real*8, dimension(:,:),   pointer    :: tab               !< tableau 2d real pointer
Character(len=10), dimension(3)      :: dimtestname       !> pour la sortie test netcdf
integer                              :: ncidloc           !> pour la sortie test netcdf
integer                              :: status            !> pour la sortie test netcdf

integer                              :: massb_time        !< pour selectionner le type de calcul de smb


contains

!--------------------------------------------------------------------------------
!> SUBROUTINE: input_clim  
!! Routine qui permet d'initialiser les variations temporelles des variables climatiques
!>
!--------------------------------------------------------------------------------

subroutine input_clim  

  use io_netcdf_grisli, only: read_ncdf_var
  use geography, only: dirnameinp,dirforcage
  use module3d_phy,only: acc,abl,BM,Tann,Tjuly,num_param,num_rep_42,num_forc

  implicit none

  character(len=100) :: smb_file           ! fichier smb
  character(len=100) :: temp_annual_file   ! temperature annuelles
  character(len=100) :: file_smb_snap      !> nom du fichier dans lequel sont les snapshots

  integer            :: err                ! recuperation d'erreur
  integer            :: i,j,k

  !aurel for Tafor:
  character(len=8) :: control      !label to check clim. forc. file (filin) is usable
  character(len=100):: filin

  namelist/clim_smb_T_gen/smb_file,coef_smb_unit,temp_annual_file

  rewind(num_param)                     ! pour revenir au debut du fichier param_list.dat
  read(num_param,clim_smb_T_gen)

  write(num_rep_42,'(A)')'!  module climat_Grice2sea_years_perturb_mod                     '
  write(num_rep_42,clim_smb_T_gen)
  write(num_rep_42,'(A)')'! smb_file          = fichier SMB (kg/m2/an)                     '
  write(num_rep_42,'(A)')'! coef_smb_unit     = coef passage m glace/an  (1/910 ou 1/918)  '
  write(num_rep_42,'(A)')'! temp_annual_file  = Temp moy annuelle  (C)                    '
  write(num_rep_42,'(A)')'!________________________________________________________________'

  
! smb : surface mass balance
  smb_file  = trim(dirnameinp)//trim(smb_file)

  call Read_Ncdf_var('smb',smb_file,tab)

!  call lect_input(3,'smb',1,bm,smb_file,trim(dirnameinp)//trim(runname)//'.nc')

  bm(:,:)  = tab(:,:) * coef_smb_unit

  !where ((H(:,:).lt.1.).and.(Bsoc(:,:).gt.0.))
  !   bm(:,:) = bm(:,:) - 5.                     ! pour faire un masque a l'exterieur du Groenland actuel
  !end where
  !where ((H(:,:).lt.1.).and.(Bsoc(:,:).gt.0.).and.(bm(:,:).gt.0.))
  !   bm(:,:) = -0.1
  !end where
  where (bm(:,:).eq.0) bm(:,:)=-5. !afq
  
  acc(:,:) = 0.
  abl(:,:) = 0.

  where (bm(:,:).gt.0.)
     acc(:,:) = bm(:,:)   ! accumulation quand positif
  elsewhere
     abl(:,:) = - bm(:,:) ! ablation quand negatif
  end where


! surface temperature  Tann

  temp_annual_file = trim(dirnameinp)//trim(temp_annual_file)

  call Read_Ncdf_var('Tann',temp_annual_file,tab)
  Tann(:,:)  = tab(:,:)

  where (Tann(:,:).lt.-100.) Tann(:,:)=10.

  ta0(:,:)   = Tann(:,:)
  Tjuly(:,:) = Tann(:,:)
  bm_0(:,:) = bm(:,:)
  
  filin=trim(dirforcage)//trim(filforc)

  open(num_forc,file=filin,status='old')
  
  read(num_forc,*) control,nft
  
    ! Determination of file size (line nb), allocation of perturbation array
  
  if (control.ne.'nb_lines') then
     write(6,*) filin,'indiquer le nb de ligne en debut de fichier:'
     write(6,*) 'le nb de lignes et le label de control nb_lines'
     stop
  endif
    
    ! Dimensionnement des tableaux tdate, ....

  if (.not.allocated(tdate)) then
     allocate(tdate(nft),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau Tdate",err
        stop 4
     end if
  end if

  if (.not.allocated(spert)) then
     allocate(spert(nft),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau Spert",err
        stop 4
     end if
  end if
  
  if (.not.allocated(tpert)) then
     allocate(tpert(nft),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau Tpert",err
        stop 4
     end if
  end if
  
  if (.not.allocated(bpert)) then
     allocate(bpert(nft),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau Bpert",err
        stop 4
     end if
  end if
  
  do i=1,nft
     if (pertbmb==1) then
        read(num_forc,*) tdate(i),spert(i),tpert(i),bpert(i)
     else
        read(num_forc,*) tdate(i),spert(i),tpert(i)
        bpert(i)=0.
     endif
  end do
  close(num_forc)
  
  tpert(:)=tpert(:)*coefT
  bpert(:)=max(1.+bpert(:)*coefbmb,0.01) !freezing is not allowed

end subroutine input_clim

!--------------------------------------------------------------------------------
!> SUBROUTINE: init_forclim
!! Routine qui permet d'initialiser les variables climatiques au cours du temps
!>
subroutine init_forclim

  use module3d_phy,only: num_param,num_rep_42

  implicit none
  namelist/lapse_rates/T_lapse_rate
  namelist/clim_pert_massb/coefT,filforc,pertsmb,rapsmb,pertbmb,coefbmb
  
  rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
  read(num_param,lapse_rates)

  write(num_rep_42,'(A)')'!  module climat_Grice2sea_years_perturb_mod                     '
  write(num_rep_42,lapse_rates)
  write(num_rep_42,'(A)')'!T_lapse_rate       = lapse rate temp annuelle                   '
  write(num_rep_42,'(A)')'!________________________________________________________________'


  rewind(num_param)  
  read(num_param,clim_pert_massb)
  
  write(num_rep_42,'(A)') '! module climat_Grice2sea_years_perturb_mod                      '
  write(num_rep_42,clim_pert_massb)
  write(num_rep_42,'(A)') '! coefT : coef amplitude perturbation T'
  write(num_rep_42,'(A)') '! filforc : fichier de forcage temporel'
  write(num_rep_42,'(A)') '! pertsmb : 0 SMB fixe, 1 SMB exp temperature'
  write(num_rep_42,'(A)') '! rapsmb : coef variation SMB'
  write(num_rep_42,'(A)') '! pertbmb : 0 bmelt fixe, 1 bmelt variable'
  write(num_rep_42,'(A)') '! coefbmb : coef variation bmelt'
  write(num_rep_42,'(A)') '!________________________________________________________________'

  return
end subroutine init_forclim

!--------------------------------------------------------------------------------
!> SUBROUTINE: forclim
!!  
!!  Routine qui permet le calcul climatique au cours du temps
!!  @note Au temps considere (time) attribue les scalaires
!!  @note   - tafor : forcage en temperature
!!  @note   - sealevel : forcage niveau des mers
!!  @note   - coefbmelt : forcage fusion basale ice shelves 
!>
subroutine forclim               !  au temps considere (time) 

  use module3d_phy,only: tafor,time,sealevel,sealevel_2d,coefbmshelf,acc,abl,BM,Tann,Tjuly,S,S0,Ts,time

  implicit none

  integer i,ift

  ! smb fixe et Tann avec lapse rate + Tafor
        
  if(time.lt.tdate(1)) then
     tafor=tpert(1)
     sealevel=spert(1)
     coefbmshelf=bpert(1)
     ift=1

  else if (time.ge.tdate(nft)) then
     tafor=tpert(nft)
     sealevel=spert(nft)
     coefbmshelf=bpert(nft)
     ift=nft
           
  else
     do i=1,nft-1
        if((time.ge.tdate(i)).and.(time.lt.tdate(i+1))) then ! entre i et i+1 : cas general
           tafor=tpert(i)+(tpert(i+1)-tpert(i))*       &
                (time-tdate(i))/(tdate(i+1)-tdate(i))
           sealevel=spert(i)+(spert(i+1)-spert(i))*    &
                (time-tdate(i))/(tdate(i+1)-tdate(i))
           coefbmshelf=bpert(i)+(bpert(i+1)-bpert(i))*    &
                (time-tdate(i))/(tdate(i+1)-tdate(i))
           ift=i
           goto 100
        endif
     end do
  endif
100 continue

  sealevel_2d(:,:) = sealevel

  Tann (:,:) = Ta0 (:,:) + T_lapse_rate * (S(:,:)-S0(:,:)) +Tafor
  Tjuly(:,:) = Tann(:,:)
  Ts(:,:)    = Tann(:,:)

  ! aurel marion dufresne: we might want to decrease the SMB during glacials..?
  if (pertsmb.eq.1) then
     bm(:,:) = bm_0(:,:) * exp( rapsmb *(Tann(:,:)-Ta0(:,:)))
     if (Tafor.lt.0.) then
        where(bm(:,:).lt.0.) bm(:,:)=min(bm(:,:)-Tafor*0.05,1.) !10 degrees less give 0.5 meter more ?
     end if
  end if

  !afq -- as in Pollard and Deconto, melt is more efficient at high temperature... ->
  if (coefbmshelf.gt.1.) then
     coefbmshelf = 1. + ( coefbmshelf - 1. ) * 1.5 !afq.... tuning for the degla?...
  end if
        
!afq --old:        ! coefmshelf est un coefficient qui fait varier bmgrz et bmshelf en fonction de tafor
!afq --old:        coefbmshelf=(1.+tafor/10.)       ! coefbmshelf=0 pour tafor=-10deg
!afq --old:        coefbmshelf=max(coefbmshelf,0.)  
!afq --old:        coefbmshelf=min(coefbmshelf,2.)
  
  where (bm(:,:).gt.0.)
     acc(:,:) = bm(:,:)   ! accumulation quand positif
  elsewhere
     abl(:,:) = - bm(:,:) ! ablation quand negatif
  end where

end subroutine forclim


end module  climat_smb_perturb_mod
